@@ -13,6 +13,7 @@ from ..models import Facility
 from ..models import FacilityUser
 from ..models import KolibriAnonymousUser
 from ..models import LearnerGroup
+from ..models import Membership
 from .helpers import create_dummy_facility_data
 from .helpers import create_superuser
 
@@ -34,31 +35,26 @@ class RolesWithinFacilityTestCase(TestCase):
         admin = self.data["facility_admin"]
         facility = self.data["facility"]
         self.assertTrue(admin.has_role_for(role_kinds.ADMIN, facility))
-        self.assertIn(role_kinds.ADMIN, admin.get_roles_for(facility))
 
     def test_coach_has_coach_role_for_own_classroom(self):
         coach0 = self.data["classroom_coaches"][0]
         classroom0 = self.data["classrooms"][0]
         self.assertTrue(coach0.has_role_for(role_kinds.COACH, classroom0))
-        self.assertIn(role_kinds.COACH, coach0.get_roles_for(classroom0))
 
     def test_coach_has_no_coach_role_for_other_classroom(self):
         coach0 = self.data["classroom_coaches"][0]
         classroom1 = self.data["classrooms"][1]
         self.assertFalse(coach0.has_role_for(role_kinds.COACH, classroom1))
-        self.assertNotIn(role_kinds.COACH, coach0.get_roles_for(classroom1))
 
     def test_coach_has_coach_role_for_learner_from_own_classroom(self):
         coach0 = self.data["classroom_coaches"][0]
         learner0 = self.data["learners_one_group"][0][0]
         self.assertTrue(coach0.has_role_for(role_kinds.COACH, learner0))
-        self.assertIn(role_kinds.COACH, coach0.get_roles_for(learner0))
 
     def test_coach_has_no_coach_role_for_learner_from_other_classroom(self):
         coach0 = self.data["classroom_coaches"][0]
         learner1 = self.data["learners_one_group"][1][0]
         self.assertFalse(coach0.has_role_for(role_kinds.COACH, learner1))
-        self.assertNotIn(role_kinds.COACH, coach0.get_roles_for(learner1))
 
 
 class ImplicitMembershipTestCase(TestCase):
@@ -74,11 +70,6 @@ class ImplicitMembershipTestCase(TestCase):
     def test_has_admin_role_for_learner(self):
         self.assertTrue(self.admin.has_role_for(role_kinds.ADMIN, self.learner))
 
-    def test_only_has_admin_role_for_learner(self):
-        self.assertEqual(
-            self.admin.get_roles_for(self.learner), set([role_kinds.ADMIN])
-        )
-
     def test_admin_can_read_learner_object(self):
         self.assertTrue(self.admin.can_read(self.learner))
 
@@ -95,23 +86,21 @@ class ExplicitMembershipTestCase(TestCase):
 
         cls.admin = FacilityUser.objects.create(username="admin", facility=cls.facility)
         cls.classroom = Classroom.objects.create(name="Class", parent=cls.facility)
-        cls.classroom.add_admin(cls.admin)
+        cls.facility.add_admin(cls.admin)
 
         cls.learner = FacilityUser.objects.create(
             username="learner", facility=cls.facility
         )
+        cls.classroom.add_member(cls.learner)
         cls.group = LearnerGroup.objects.create(name="Group", parent=cls.classroom)
         cls.group.add_member(cls.learner)
 
     def test_has_admin_role_for_learner(self):
+        # We do not support classroom admin roles
         self.assertTrue(self.admin.has_role_for(role_kinds.ADMIN, self.learner))
 
-    def test_only_has_admin_role_for_learner(self):
-        self.assertEqual(
-            self.admin.get_roles_for(self.learner), set([role_kinds.ADMIN])
-        )
-
     def test_admin_can_read_learner_object(self):
+        # We do not support classroom admin roles
         self.assertTrue(self.admin.can_read(self.learner))
 
     def test_learner_is_in_list_of_readable_objects(self):
@@ -119,36 +108,11 @@ class ExplicitMembershipTestCase(TestCase):
             self.learner, self.admin.filter_readable(FacilityUser.objects.all())
         )
 
-
-class RolesAcrossFacilitiesTestCase(TestCase):
-    @classmethod
-    def setUpTestData(cls):
-        cls.data1 = create_dummy_facility_data()
-        cls.data2 = create_dummy_facility_data()
-
-    def test_no_roles_between_users_across_facilities(self):
-        users1 = self.data1["all_users"]
-        users2 = self.data2["all_users"]
-        for user1 in users1:
-            for user2 in users2:
-                if not user1.is_superuser:
-                    self.assertEqual(len(user1.get_roles_for(user2)), 0)
-
-    def test_no_roles_for_collections_across_facilities(self):
-        users1 = (
-            self.data1["classroom_coaches"]
-            + [self.data1["facility_admin"]]
-            + list(self.data1["facility"].get_members())
+    def test_learnergroup_membership_gets_deleted(self):
+        self.classroom.remove_member(self.learner)
+        self.assertFalse(
+            Membership.objects.filter(collection=self.group, user=self.learner).exists()
         )
-        collections2 = (
-            [self.data2["facility"]]
-            + self.data2["classrooms"]
-            + flatten(self.data2["learnergroups"])
-        )
-        for user1 in users1:
-            for collection2 in collections2:
-                if not user1.is_superuser:
-                    self.assertEqual(len(user1.get_roles_for(collection2)), 0)
 
 
 class MembershipWithinFacilityTestCase(TestCase):
@@ -164,7 +128,6 @@ class MembershipWithinFacilityTestCase(TestCase):
             + self.data["unattached_users"]
             + [self.data["facility_admin"]]
             + [self.data["facility_coach"]]
-            + self.data["classroom_admins"]
             + self.data["classroom_coaches"]
             + [self.data["superuser"]]
         )
@@ -256,14 +219,16 @@ class AnonymousUserRolesTestCase(TestCase):
     def test_anon_user_has_no_admin_role_for_anyone(self):
         for user in self.data["all_users"]:
             self.assertFalse(self.anon_user.has_role_for(role_kinds.ADMIN, user))
-            self.assertEqual(len(self.anon_user.get_roles_for(user)), 0)
 
     def test_anon_user_has_no_admin_role_for_any_collection(self):
         for coll in self.data["all_collections"]:
             self.assertFalse(self.anon_user.has_role_for(role_kinds.ADMIN, coll))
-            self.assertEqual(len(self.anon_user.get_roles_for(coll)), 0)
 
     def test_nobody_but_superuser_has_roles_for_anon_user(self):
         for user in self.data["all_users"]:
             if not user.is_superuser:
-                self.assertEqual(len(user.get_roles_for(self.anon_user)), 0)
+                self.assertFalse(
+                    user.has_role_for(
+                        [role_kinds.ADMIN, role_kinds.COACH], self.anon_user
+                    )
+                )
