@@ -14,7 +14,7 @@ from kolibri.core.auth.constants.collection_kinds import ADHOCLEARNERSGROUP
 from kolibri.core.auth.models import Collection
 from kolibri.core.auth.models import FacilityUser
 from kolibri.core.auth.models import Membership
-from kolibri.core.auth.utils import create_adhoc_group_for_learners
+from kolibri.core.auth.utils.users import create_adhoc_group_for_learners
 from kolibri.core.exams.models import Exam
 from kolibri.core.exams.models import ExamAssignment
 from kolibri.core.serializers import DateTimeTzField
@@ -28,7 +28,8 @@ class NestedCollectionSerializer(ModelSerializer):
 
 class QuestionSourceSerializer(Serializer):
     exercise_id = HexUUIDField(format="hex")
-    question_id = CharField()
+    # V0 need not have question_id that is why required=False
+    question_id = HexUUIDField(format="hex", required=False)
     title = CharField()
     counter_in_exercise = IntegerField()
 
@@ -36,9 +37,7 @@ class QuestionSourceSerializer(Serializer):
 class ExamSerializer(ModelSerializer):
 
     assignments = ListField(
-        child=PrimaryKeyRelatedField(
-            read_only=False, queryset=Collection.objects.all()
-        ),
+        child=PrimaryKeyRelatedField(read_only=False, queryset=Collection.objects.all())
     )
     learner_ids = ListField(
         child=PrimaryKeyRelatedField(
@@ -154,7 +153,7 @@ class ExamSerializer(ModelSerializer):
                     collection__kind=ADHOCLEARNERSGROUP
                 ).values_list("collection__id", flat=True)
             )
-            new_group_ids = set(x.id for x in collections)
+            new_group_ids = {x.id for x in collections}
 
             for cid in new_group_ids - current_group_ids:
                 collection = Collection.objects.get(id=cid)
@@ -200,7 +199,7 @@ class ExamSerializer(ModelSerializer):
                     collection=adhoc_group_assignment.collection
                 ).values_list("user_id", flat=True)
                 original_learner_ids_set = set(original_learner_ids)
-                learner_ids_set = set(learner.id for learner in learners)
+                learner_ids_set = {learner.id for learner in learners}
                 if original_learner_ids_set != learner_ids_set:
                     # Only bother to do anything if these are different
                     new_learner_ids = learner_ids_set - original_learner_ids_set
