@@ -3,7 +3,7 @@ import ntpath
 import os
 import shutil
 from functools import partial
-from tempfile import NamedTemporaryFile
+from tempfile import mkstemp
 
 import requests
 from django.apps.registry import AppRegistryNotReady
@@ -22,6 +22,8 @@ from rest_framework import decorators
 from rest_framework import serializers
 from rest_framework import status
 from rest_framework import viewsets
+from rest_framework.authentication import BasicAuthentication
+from rest_framework.authentication import SessionAuthentication
 from rest_framework.exceptions import APIException
 from rest_framework.exceptions import AuthenticationFailed
 from rest_framework.exceptions import ParseError
@@ -34,7 +36,7 @@ from .permissions import FacilitySyncPermissions
 from kolibri.core.auth.constants.morango_sync import PROFILE_FACILITY_DATA
 from kolibri.core.auth.constants.morango_sync import State as FacilitySyncState
 from kolibri.core.auth.management.utils import get_client_and_server_certs
-from kolibri.core.auth.management.utils import get_dataset_id
+from kolibri.core.auth.management.utils import get_facility_dataset_id
 from kolibri.core.auth.models import Facility
 from kolibri.core.content.models import ChannelMetadata
 from kolibri.core.content.permissions import CanExportLogs
@@ -54,9 +56,12 @@ from kolibri.core.discovery.utils.network.errors import NetworkLocationNotFound
 from kolibri.core.discovery.utils.network.errors import URLParseError
 from kolibri.core.logger.csv_export import CSV_EXPORT_FILENAMES
 from kolibri.core.tasks.exceptions import JobNotFound
+from kolibri.core.tasks.exceptions import JobNotRestartable
 from kolibri.core.tasks.exceptions import UserCancelledError
+from kolibri.core.tasks.job import JobRegistry
 from kolibri.core.tasks.job import State
 from kolibri.core.tasks.main import facility_queue
+from kolibri.core.tasks.main import job_storage
 from kolibri.core.tasks.main import priority_queue
 from kolibri.core.tasks.main import queue
 from kolibri.core.tasks.utils import get_current_job
@@ -181,23 +186,56 @@ class BaseViewSet(viewsets.ViewSet):
     queues = []
     permission_classes = []
 
+    # Adding auth classes explicitly until we find a fix for BasicAuth not
+    # working on tasks API (in dev settings)
+    authentication_classes = [SessionAuthentication, BasicAuthentication]
+
     def initial(self, request, *args, **kwargs):
         if len(self.permission_classes) == 0:
             self.permission_classes = self.default_permission_classes()
+        if self.permission_classes is None:
+            self.permission_classes = []
         return super(BaseViewSet, self).initial(request, *args, **kwargs)
 
     def default_permission_classes(self):
-        # task permissions shared between facility management and device management
-        if self.action in ["list", "deletefinishedtasks"]:
-            return [CanManageContent | CanExportLogs]
-        elif self.action == "startexportlogcsv":
-            return [CanExportLogs]
-        elif self.action in ["importusersfromcsv", "exportuserstocsv"]:
-            return [CanImportUsers]
-
-        # this was the default before, so leave as is for any other endpoints
+        # For all /api/tasks/ endpoints
         return [CanManageContent]
 
+    def validate_create_req_data(self, request):
+        """
+        Validates the request data received on POST /api/tasks/.
+
+        If `request.user` is authorized to initiate the `task` function, this returns
+        a list of `request.data` otherwise raises PermissionDenied.
+        """
+        if isinstance(request.data, list):
+            request_data_list = request.data
+        else:
+            request_data_list = [request.data]
+
+        for request_data in request_data_list:
+            if "task" not in request_data:
+                raise serializers.ValidationError("The 'task' field is required.")
+            if not isinstance(request_data["task"], string_types):
+                raise serializers.ValidationError("The 'task' value must be a string.")
+
+            funcstr = request_data.get("task")
+
+            # Make sure the task is registered
+            try:
+                registered_job = JobRegistry.REGISTERED_JOBS[funcstr]
+            except KeyError:
+                raise serializers.ValidationError(
+                    "'{funcstr}' is not registered.".format(funcstr=funcstr)
+                )
+
+            # Check permissions the DRF way
+            for permission in registered_job.permissions:
+                if not permission.has_permission(request, self):
+                    self.permission_denied(request)
+
+        return request_data_list
+
     def list(self, request):
         jobs_response = [
             _job_to_response(j) for _queue in self.queues for j in _queue.jobs
@@ -206,8 +244,59 @@ class BaseViewSet(viewsets.ViewSet):
         return Response(jobs_response)
 
     def create(self, request):
-        # unimplemented. Call out to the task-specific APIs for now.
-        pass
+        """
+        Enqueue a task for async processing.
+
+        API endpoint:
+            POST /api/tasks/
+
+        Request payload parameters:
+            - `task` (required): a string representing the dotted path to task function.
+            - all other key value pairs are passed to the validator if the
+              task function has one otherwise they are passed to the task function itself
+              as keyword args.
+
+        Keep in mind:
+            If a task function has a validator then dict returned by the validator
+            is passed to the task function as keyword args.
+
+            The validator can add `extra_metadata` in the returning dict to set `extra_metadata`
+            in the enqueued task.
+        """
+        request_data_list = self.validate_create_req_data(request)
+
+        enqueued_jobs_response = []
+
+        # Once we have validated all the tasks, we are good to go!
+        for request_data in request_data_list:
+
+            funcstr = request_data.pop("task")
+            registered_job = JobRegistry.REGISTERED_JOBS[funcstr]
+            # Run validator with request and request_data as its argument
+            if registered_job.validator is not None:
+                try:
+                    validator_result = registered_job.validator(request, request_data)
+                except Exception as e:
+                    raise e
+
+                if not isinstance(validator_result, dict):
+                    raise serializers.ValidationError("Validator must return a dict.")
+
+                extra_metadata = validator_result.get("extra_metadata")
+                if extra_metadata is not None and not isinstance(extra_metadata, dict):
+                    raise serializers.ValidationError(
+                        "In the dict returned by validator, 'extra_metadata' must be a dict."
+                    )
+
+                request_data = validator_result
+
+            job_id = registered_job.enqueue(**request_data)
+            enqueued_jobs_response.append(_job_to_response(job_storage.get_job(job_id)))
+
+        if len(enqueued_jobs_response) == 1:
+            enqueued_jobs_response = enqueued_jobs_response[0]
+
+        return Response(enqueued_jobs_response)
 
     def retrieve(self, request, pk=None):
         for _queue in self.queues:
@@ -221,6 +310,30 @@ class BaseViewSet(viewsets.ViewSet):
 
         return Response(task)
 
+    @decorators.action(methods=["post"], detail=False)
+    def restarttask(self, request):
+        """
+        Restart a task with its task id given in the task_id parameter.
+        """
+
+        if "task_id" not in request.data:
+            raise serializers.ValidationError("The 'task_id' field is required.")
+        if not isinstance(request.data["task_id"], string_types):
+            raise serializers.ValidationError("The 'task_id' should be a string.")
+
+        resp = {}
+        for _queue in self.queues:
+            try:
+                task_id = _queue.restart_job(request.data["task_id"])
+                resp = _job_to_response(_queue.fetch_job(task_id))
+                break
+            except JobNotFound:
+                continue
+            except JobNotRestartable as e:
+                raise serializers.ValidationError(str(e))
+
+        return Response(resp)
+
     def destroy(self, request, pk=None):
         # unimplemented for now.
         pass
@@ -284,14 +397,20 @@ class BaseViewSet(viewsets.ViewSet):
 
 
 class TasksViewSet(BaseViewSet):
-    queues = [queue, priority_queue]
+    @property
+    def queues(self):
+        return [queue, priority_queue]
 
     def default_permission_classes(self):
-        # exclusive permission for facility management
-        if self.action == "startexportlogcsv":
+        if self.action in ["list", "deletefinishedtasks"]:
+            return [CanManageContent | CanExportLogs]
+        elif self.action == "startexportlogcsv":
             return [CanExportLogs]
+        elif self.action in ["importusersfromcsv", "exportuserstocsv"]:
+            return [CanImportUsers]
 
-        return super(TasksViewSet, self).permission_classes
+        # For all other tasks
+        return [CanManageContent]
 
     @decorators.action(methods=["post"], detail=False)
     def startchannelupdate(self, request):
@@ -595,7 +714,8 @@ class TasksViewSet(BaseViewSet):
         drives = get_mounted_drives_with_channel_info()
 
         # make sure everything is a dict, before converting to JSON
-        assert isinstance(drives, dict)
+        if not isinstance(drives, dict):
+            raise AssertionError
         out = [mountdata._asdict() for mountdata in drives.values()]
 
         return Response(out)
@@ -616,9 +736,8 @@ class TasksViewSet(BaseViewSet):
             # Django uses InMemoryUploadedFile for files less than 2.5Mb
             # and TemporaryUploadedFile for bigger files:
             if type(upload.file) == InMemoryUploadedFile:
-                with NamedTemporaryFile(
-                    dir=temp_dir, suffix=".upload", delete=False
-                ) as dest:
+                _, filepath = mkstemp(dir=temp_dir, suffix=".upload")
+                with open(filepath, "w+b") as dest:
                     filepath = dest.name
                     for chunk in upload.file.chunks():
                         dest.write(chunk)
@@ -808,7 +927,7 @@ class TasksViewSet(BaseViewSet):
             channel_metadata = read_channel_metadata_from_db_file(
                 get_content_database_file_path(channel_id, drive.datafolder)
             )
-            job_metadata["new_channel_version"] = channel_metadata.version
+            job_metadata["new_channel_version"] = channel_metadata["version"]
         else:
             raise serializers.ValidationError(
                 "'method' field should either be 'network' or 'disk'."
@@ -839,16 +958,13 @@ class TasksViewSet(BaseViewSet):
 
 
 class FacilityTasksViewSet(BaseViewSet):
-    queues = [facility_queue]
+    @property
+    def queues(self):
+        return [facility_queue]
 
     def default_permission_classes(self):
-        permission_classes = super(FacilityTasksViewSet, self).permission_classes
-
         if self.action in ["list", "retrieve"]:
-            return [p | FacilitySyncPermissions for p in permission_classes]
-
-        # All other permissions are deferred to permission_classes decorator
-        return []
+            return [FacilitySyncPermissions]
 
     @decorators.action(
         methods=["post"], detail=False, permission_classes=[FacilitySyncPermissions]
@@ -857,8 +973,11 @@ class FacilityTasksViewSet(BaseViewSet):
         """
         Initiate a PUSH sync with Kolibri Data Portal.
         """
-        job_data = validate_prepare_sync_job(
-            request, extra_metadata=prepare_sync_task(request, type="SYNCDATAPORTAL")
+        facility_id = validate_facility(request)
+        sync_args = validate_sync_task(request)
+        job_data = prepare_sync_job(
+            facility=facility_id,
+            extra_metadata=prepare_sync_task(*sync_args, type="SYNCDATAPORTAL"),
         )
         job_id = facility_queue.enqueue(call_command, "sync", **job_data)
 
@@ -891,11 +1010,16 @@ class FacilityTasksViewSet(BaseViewSet):
         """
         Initiate a PULL of a specific facility from another device.
         """
-        job_data = validate_and_prepare_peer_sync_job(
-            request,
+
+        baseurl, facility_id, username, password = validate_peer_sync_job(request)
+        validate_and_create_sync_credentials(baseurl, facility_id, username, password)
+        sync_args = validate_sync_task(request)
+        job_data = prepare_peer_sync_job(
+            baseurl,
+            facility_id,
             no_push=True,
             no_provision=True,
-            extra_metadata=prepare_sync_task(request, type="SYNCPEER/PULL"),
+            extra_metadata=prepare_sync_task(*sync_args, type="SYNCPEER/PULL"),
         )
 
         job_id = facility_queue.enqueue(call_command, "sync", **job_data)
@@ -910,8 +1034,13 @@ class FacilityTasksViewSet(BaseViewSet):
         """
         Initiate a SYNC (PULL + PUSH) of a specific facility from another device.
         """
-        job_data = validate_and_prepare_peer_sync_job(
-            request, extra_metadata=prepare_sync_task(request, type="SYNCPEER/FULL")
+        baseurl, facility_id, username, password = validate_peer_sync_job(request)
+        validate_and_create_sync_credentials(baseurl, facility_id, username, password)
+        sync_args = validate_sync_task(request)
+        job_data = prepare_peer_sync_job(
+            baseurl,
+            facility_id,
+            extra_metadata=prepare_sync_task(*sync_args, type="SYNCPEER/FULL"),
         )
         job_id = facility_queue.enqueue(call_command, "sync", **job_data)
 
@@ -980,12 +1109,20 @@ class ResourceGoneError(APIException):
     default_detail = "Unable to connect"
 
 
-def prepare_sync_task(request, **kwargs):
-    facility_id = request.data.get("facility")
+def prepare_sync_task(
+    facility_id,
+    user_id,
+    username,
+    facility_name,
+    device_name,
+    device_id,
+    baseurl,
+    **kwargs
+):
     task_data = dict(
         facility=facility_id,
-        started_by=request.user.pk,
-        started_by_username=request.user.username,
+        started_by=user_id,
+        started_by_username=username,
         sync_state=FacilitySyncState.PENDING,
         bytes_sent=0,
         bytes_received=0,
@@ -995,22 +1132,22 @@ def prepare_sync_task(request, **kwargs):
     if task_type in ["SYNCPEER/PULL", "SYNCPEER/FULL"]:
         # Extra metadata that can be passed from the client
         extra_task_data = dict(
-            facility_name=request.data.get("facility_name", ""),
-            device_name=request.data.get("device_name", ""),
-            device_id=request.data.get("device_id", ""),
-            baseurl=request.data.get("baseurl", ""),
+            facility_name=facility_name,
+            device_name=device_name,
+            device_id=device_id,
+            baseurl=baseurl,
         )
         task_data.update(extra_task_data)
     elif task_type == "SYNCDATAPORTAL":
         # Extra metadata that can be passed from the client
-        extra_task_data = dict(facility_name=request.data.get("facility_name", ""))
+        extra_task_data = dict(facility_name=facility_name)
         task_data.update(extra_task_data)
 
     task_data.update(kwargs)
     return task_data
 
 
-def validate_prepare_sync_job(request, **kwargs):
+def validate_facility(request):
     # ensure we have the facility
     try:
         facility_id = request.data.get("facility")
@@ -1019,11 +1156,33 @@ def validate_prepare_sync_job(request, **kwargs):
     except KeyError:
         raise ParseError("Missing `facility` parameter")
 
+    return facility_id
+
+
+def validate_sync_task(request):
+    facility_id = validate_facility(request)
+    user_id = request.user.pk
+    username = request.user.username
+    facility_name = request.data.get("facility_name", "")
+    device_name = request.data.get("device_name", "")
+    device_id = request.data.get("device_id", "")
+    baseurl = request.data.get("baseurl", "")
+    return (
+        facility_id,
+        user_id,
+        username,
+        facility_name,
+        device_name,
+        device_id,
+        baseurl,
+    )
+
+
+def prepare_sync_job(**kwargs):
     job_data = dict(
-        facility=facility_id,
         chunk_size=200,
         noninteractive=True,
-        extra_metadata=dict(),
+        extra_metadata={},
         track_progress=True,
         cancellable=False,
     )
@@ -1032,7 +1191,7 @@ def validate_prepare_sync_job(request, **kwargs):
     return job_data
 
 
-def validate_and_prepare_peer_sync_job(request, **kwargs):
+def validate_peer_sync_job(request):
     # validate the baseurl
     try:
         address = request.data.get("baseurl")
@@ -1047,12 +1206,23 @@ def validate_and_prepare_peer_sync_job(request, **kwargs):
     except NetworkLocationNotFound:
         raise ResourceGoneError()
 
-    job_data = validate_prepare_sync_job(request, baseurl=baseurl, **kwargs)
+    facility_id = validate_facility(request)
 
-    facility_id = job_data.get("facility")
     username = request.data.get("username", None)
     password = request.data.get("password", None)
 
+    return (baseurl, facility_id, username, password)
+
+
+def validate_and_create_sync_credentials(
+    baseurl, facility_id, username, password, user_id=None
+):
+    """
+    Validates user credentials for syncing by performing certificate verification, which will also
+    save any certificates after successful authentication
+
+    :param user_id: Optional user ID for SoUD use case
+    """
     # call this in case user directly syncs without migrating database
     if not ScopeDefinition.objects.filter():
         call_command("loaddata", "scopedefinitions")
@@ -1063,13 +1233,19 @@ def validate_and_prepare_peer_sync_job(request, **kwargs):
     # try to get the certificate, which will save it if successful
     try:
         # make sure we get the dataset ID
-        dataset_id = get_dataset_id(
+        facility_id, dataset_id = get_facility_dataset_id(
             baseurl, identifier=facility_id, noninteractive=True
         )
 
         # username and password are not required for this to succeed unless there is no cert
         get_client_and_server_certs(
-            username, password, dataset_id, network_connection, noninteractive=True
+            username,
+            password,
+            dataset_id,
+            network_connection,
+            user_id=user_id,
+            facility_id=facility_id,
+            noninteractive=True,
         )
     except (CommandError, HTTPError) as e:
         if not username and not password:
@@ -1077,7 +1253,33 @@ def validate_and_prepare_peer_sync_job(request, **kwargs):
         else:
             raise AuthenticationFailed(e)
 
-    return job_data
+
+def prepare_peer_sync_job(baseurl, facility_id, **kwargs):
+    """
+    Initializes and validates connection to peer with username and password for the sync command. If
+    already initialized, the username and password do not need to be supplied
+    """
+    return prepare_sync_job(facility=facility_id, baseurl=baseurl, **kwargs)
+
+
+def prepare_soud_sync_job(baseurl, facility_id, user_id, **kwargs):
+    """
+    A SoUD sync requires that the device is already "registered" with the server, so there
+    shouldn't be a need for username/password and the verification of those. This eliminates the
+    validation to keep overhead low for automated single-user syncing. To initialize with a peer
+    for a SoUD, use `prepare_peer_sync_job` with `user` keyword argument
+    """
+    return prepare_sync_job(
+        baseurl=baseurl, facility=facility_id, user=user_id, **kwargs
+    )
+
+
+def prepare_soud_resume_sync_job(baseurl, sync_session_id, user_id, **kwargs):
+    """
+    Resuming a SoUD sync requires that a normal sync has occurred and the `SyncSession` is still
+    active
+    """
+    return prepare_sync_job(baseurl=baseurl, id=sync_session_id, user=user_id, **kwargs)
 
 
 def _remoteimport(
@@ -1231,15 +1433,14 @@ def _job_to_response(job):
             "cancellable": False,
             "clearable": False,
         }
-    else:
-        output = {
-            "status": job.state,
-            "exception": str(job.exception),
-            "traceback": str(job.traceback),
-            "percentage": job.percentage_progress,
-            "id": job.job_id,
-            "cancellable": job.cancellable,
-            "clearable": job.state in [State.FAILED, State.CANCELED, State.COMPLETED],
-        }
-        output.update(job.extra_metadata)
-        return output
+    output = {
+        "status": job.state,
+        "exception": str(job.exception),
+        "traceback": str(job.traceback),
+        "percentage": job.percentage_progress,
+        "id": job.job_id,
+        "cancellable": job.cancellable,
+        "clearable": job.state in [State.FAILED, State.CANCELED, State.COMPLETED],
+    }
+    output.update(job.extra_metadata)
+    return output
