@@ -1,63 +1,52 @@
 import logging
 import traceback
-
 from concurrent.futures import CancelledError
 
-from kolibri.core.tasks.compat import MULTIPROCESS
-from kolibri.core.tasks.exceptions import UserCancelledError
+from kolibri.core.tasks.compat import PoolExecutor
+from kolibri.core.tasks.job import execute_job
+from kolibri.core.tasks.job import Priority
 from kolibri.core.tasks.storage import Storage
 from kolibri.core.tasks.utils import InfiniteLoopThread
 
 logger = logging.getLogger(__name__)
 
 
-class Empty(Exception):
-    # An exception to raise when there are now queued jobs waiting to be started.
-    pass
-
-
 class Worker(object):
-    def __init__(self, queues, connection=None, num_workers=3):
+    def __init__(self, connection, regular_workers=2, high_workers=1):
         # Internally, we use concurrent.future.Future to run and track
         # job executions. We need to keep track of which future maps to which
         # job they were made from, and we use the job_future_mapping dict to do
         # so.
-        if connection is None:
-            raise ValueError("Connection must be defined")
 
-        # Queues that this worker executes tasks for
-        if type(queues) is not list:
-            queues = [queues]
-        self.queues = queues
         # Key: future object, Value: job object
         self.job_future_mapping = {}
+
         # Key: job_id, Value: future object
         self.future_job_mapping = {}
-        self.storage = Storage(connection)
-        self.num_workers = num_workers
 
-        self.workers = self.start_workers(num_workers=self.num_workers)
+        self.storage = Storage(connection)
+
+        # Regular workers run both 'high' and 'regular' priority jobs.
+        # High workers run only 'high' priority jobs.
+        self.regular_workers = regular_workers
+        self.max_workers = regular_workers + high_workers
+
+        self.workers = self.start_workers()
         self.job_checker = self.start_job_checker()
 
     def shutdown_workers(self, wait=True):
         # First cancel all running jobs
-        for job_id in self.future_job_mapping:
+        # Coerce to a list, as otherwise the iterable can change size
+        # during iteration, as jobs are cancelled and removed from the mapping
+        job_ids = list(self.future_job_mapping.keys())
+        for job_id in job_ids:
             logger.info("Canceling job id {}.".format(job_id))
             self.cancel(job_id)
         # Now shutdown the workers
         self.workers.shutdown(wait=wait)
 
-    def start_workers(self, num_workers):
-        if MULTIPROCESS:
-            from concurrent.futures import ProcessPoolExecutor
-
-            worker_executor = ProcessPoolExecutor
-        else:
-            from concurrent.futures import ThreadPoolExecutor
-
-            worker_executor = ThreadPoolExecutor
-
-        pool = worker_executor(max_workers=num_workers)
+    def start_workers(self):
+        pool = PoolExecutor(max_workers=self.max_workers)
         return pool
 
     def handle_finished_future(self, future):
@@ -74,12 +63,16 @@ class Worker(object):
             self.report_cancelled(job.job_id)
             return
         except Exception as e:
-            self.report_error(job.job_id, e, e.traceback)
+            if hasattr(e, "traceback"):
+                traceback = e.traceback
+            else:
+                traceback = ""
+            self.report_error(job.job_id, e, traceback)
             return
 
         self.report_success(job.job_id, result)
 
-    def shutdown(self, wait=False):
+    def shutdown(self, wait=True):
         logger.info("Asking job schedulers to shut down.")
         self.job_checker.stop()
         self.shutdown_workers(wait=wait)
@@ -93,23 +86,23 @@ class Worker(object):
         Returns: the Thread object.
         """
         t = InfiniteLoopThread(
-            self.check_jobs, thread_name="JOBCHECKER", wait_between_runs=0.5
+            self.check_jobs, thread_name="JOBCHECKER", wait_between_runs=0.2
         )
         t.start()
         return t
 
     def check_jobs(self):
         """
-        Check how many workers are currently running.
-        If fewer workers are running than there are available workers, start a new job!
+        Checks for the next job to run and also checks for jobs that should be cancelled.
+
         Returns: None
         """
-        try:
-            while len(self.future_job_mapping) < self.num_workers:
-                self.start_next_job()
-        except Empty:
-            logger.debug("No jobs to start.")
-        for job in self.storage.get_canceling_jobs(self.queues):
+        job_to_start = self.get_next_job()
+        while job_to_start:
+            self.start_next_job(job_to_start)
+            job_to_start = self.get_next_job()
+
+        for job in self.storage.get_canceling_jobs():
             job_id = job.job_id
             if job_id in self.future_job_mapping:
                 self.cancel(job_id)
@@ -120,7 +113,7 @@ class Worker(object):
         self.storage.mark_job_as_canceled(job_id)
 
     def report_success(self, job_id, result):
-        self.storage.complete_job(job_id)
+        self.storage.complete_job(job_id, result=result)
 
     def report_error(self, job_id, exc, trace):
         trace = traceback.format_exc()
@@ -130,27 +123,54 @@ class Worker(object):
     def update_progress(self, job_id, progress, total_progress, stage=""):
         self.storage.update_job_progress(job_id, progress, total_progress)
 
-    def start_next_job(self):
+    def get_next_job(self):
+        """
+        Fetches the next potential QUEUED job.
+
+        If less workers are running than there are regular workers, we look first for
+        jobs with 'high' priority, if found one we run it else we look for jobs with 'regular'
+        priority, if found we run it.
+
+        If all regular workers are busy, then the remaining workers only look for
+        'high' priority jobs. If found one, we run it.
+
+        This algorithm will make sure 'high' jobs don't wait :)
+
+        Returns the job object if a job is available based on the above algorithm else None.
+        """
+        job = None
+        workers_currently_busy = len(self.future_job_mapping)
+
+        if workers_currently_busy < self.regular_workers:
+            job = self.storage.get_next_queued_job()
+        elif workers_currently_busy < self.max_workers:
+            job = self.storage.get_next_queued_job(priority_order=[Priority.HIGH])
+        else:
+            logger.debug("All workers busy.")
+            return None
+
+        return job
+
+    def start_next_job(self, job):
         """
         start the next scheduled job to the type of workers spawned by self.start_workers.
 
         :return future:
         """
-        job = self.storage.get_next_queued_job(self.queues)
-
-        if not job:
-            raise Empty
-
         self.storage.mark_job_as_running(job.job_id)
 
-        lambda_to_execute = _reraise_with_traceback(job.get_lambda_to_execute())
+        db_type_lookup = {
+            "sqlite": "sqlite",
+            "postgresql": "postgres",
+        }
+
+        db_type = db_type_lookup[self.storage.engine.dialect.name]
 
         future = self.workers.submit(
-            lambda_to_execute,
-            update_progress_func=self.update_progress,
-            cancel_job_func=self._check_for_cancel,
-            save_job_meta_func=self.storage.save_job_meta,
-            save_as_cancellable_func=self.storage.save_job_as_cancellable,
+            execute_job,
+            job_id=job.job_id,
+            db_type=db_type,
+            db_url=self.storage.engine.url,
         )
 
         # assign the futures to a dict, mapping them to a job
@@ -169,55 +189,21 @@ class Worker(object):
         a special variable inside the future that we can check
         inside a special check_for_cancel function passed to the
         job.
+
         :param job_id:
         :return:
         """
-        future = self.future_job_mapping[job_id]
-        is_future_cancelled = future.cancel()
+        try:
+            future = self.future_job_mapping[job_id]
+            is_future_cancelled = future.cancel()
+        except KeyError:
+            # In the case that the future does not even exist, say it has been cancelled.
+            is_future_cancelled = True
 
         if is_future_cancelled:  # success!
             return True
-        else:
-            if future.running():
-                # Already running, so we manually mark the future as cancelled
-                setattr(future, "_is_cancelled", True)
-                return False
-            else:  # probably finished already, too late to cancel!
-                return False
-
-    def _check_for_cancel(self, job_id):
-        """
-        Check if a job has been requested to be cancelled. When called, the calling function can
-        optionally give the stage it is currently in, so the user has information on where the job
-        was before it was cancelled.
-
-        :param job_id: The job_id to check
-        :return: raises a UserCancelledError if we find out that we were cancelled.
-        """
-
-        future = self.future_job_mapping[job_id]
-
-        if getattr(future, "_is_cancelled", False):
-            raise UserCancelledError()
-
-
-def _reraise_with_traceback(f):
-    """
-    Call the function normally. But if the function raises an error, attach the str(traceback)
-    into the function.traceback attribute, then reraise the error.
-    Args:
-        f: The function to run.
-
-    Returns: A function that wraps f, attaching the traceback if an error occurred.
-
-    """
-
-    def wrap(*args, **kwargs):
-        try:
-            return f(*args, **kwargs)
-        except Exception as e:
-            traceback_str = traceback.format_exc()
-            e.traceback = traceback_str
-            raise
-
-    return wrap
+        if future.running():
+            # Already running, so we manually mark the future as cancelled
+            setattr(future, "_is_cancelled", True)
+            return False
+        return False
