@@ -2,10 +2,7 @@ from django.db import connections
 from django.db.models import Count
 from django.db.models import F
 from django.db.models import Max
-from django.db.models import OuterRef
 from django.db.models import Q
-from django.db.models import Subquery
-from django.db.models import Sum
 from django.db.utils import OperationalError
 from django.shortcuts import get_object_or_404
 from le_utils.constants import content_kinds
@@ -22,10 +19,12 @@ from kolibri.core.content.models import ContentNode
 from kolibri.core.exams.models import Exam
 from kolibri.core.lessons.models import Lesson
 from kolibri.core.logger import models as logger_models
+from kolibri.core.logger.utils.quiz import annotate_response_summary
 from kolibri.core.notifications.models import LearnerProgressNotification
 from kolibri.core.notifications.models import NotificationEventType
 from kolibri.core.query import annotate_array_aggregate
 from kolibri.core.sqlite.utils import repair_sqlite_db
+from kolibri.deployment.default.sqlite_db_names import NOTIFICATIONS
 
 
 # Intended to match  NotificationEventType
@@ -91,7 +90,7 @@ def content_status_serializer(lesson_data, learners_data, classroom):  # noqa C9
         }
     except OperationalError:
         notifications = []
-        repair_sqlite_db(connections["notifications_db"])
+        repair_sqlite_db(connections[NOTIFICATIONS])
 
     # In case a previously flagged learner has since completed an exercise, check all the completed
     # notifications also
@@ -146,8 +145,9 @@ def content_status_serializer(lesson_data, learners_data, classroom):  # noqa C9
 
 
 def _map_exam_status(item):
-    closed = item.pop("closed")
-    item["status"] = COMPLETED if closed else STARTED
+    complete = item.pop("complete")
+    item["status"] = COMPLETED if complete else STARTED
+    item["exam_id"] = item.pop("summarylog__content_id")
     return item
 
 
@@ -155,30 +155,11 @@ def serialize_exam_status(queryset):
     return list(
         map(
             _map_exam_status,
-            queryset.annotate(
-                last_activity=Max("attemptlogs__end_timestamp"),
-                num_correct=Subquery(
-                    logger_models.ExamAttemptLog.objects.filter(examlog=OuterRef("id"))
-                    .order_by()
-                    .values_list("item", "content_id")
-                    .distinct()
-                    .values("examlog")
-                    .annotate(total_correct=Sum("correct"))
-                    .values("total_correct")
-                ),
-                num_answered=Subquery(
-                    logger_models.ExamAttemptLog.objects.filter(examlog=OuterRef("id"))
-                    .order_by()
-                    .values_list("item", "content_id")
-                    .distinct()
-                    .values("examlog")
-                    .annotate(total_complete=Count("id"))
-                    .values("total_complete")
-                ),
-            )
+            annotate_response_summary(queryset)
+            .annotate(last_activity=Max("attemptlogs__end_timestamp"))
             .values(
-                "exam_id",
-                "closed",
+                "summarylog__content_id",
+                "complete",
                 "last_activity",
                 "num_correct",
                 "num_answered",
@@ -284,8 +265,8 @@ class ClassSummaryViewSet(viewsets.ViewSet):
         query_learners = FacilityUser.objects.filter(memberships__collection=classroom)
         query_lesson = Lesson.objects.filter(collection=pk)
         query_exams = Exam.objects.filter(collection=pk)
-        query_exam_logs = logger_models.ExamLog.objects.filter(
-            exam__in=query_exams
+        query_exam_mastery_logs = logger_models.MasteryLog.objects.filter(
+            summarylog__content_id__in=query_exams.values("id")
         ).order_by()
 
         lesson_data = serialize_lessons(query_lesson)
@@ -363,7 +344,7 @@ class ClassSummaryViewSet(viewsets.ViewSet):
                 classroom.get_individual_learners_group()
             ),
             "exams": exam_data,
-            "exam_learner_status": serialize_exam_status(query_exam_logs),
+            "exam_learner_status": serialize_exam_status(query_exam_mastery_logs),
             "content": query_content.values(
                 "content_id", "title", "kind", "channel_id", node_id=F("id")
             ),
